/* -*- mode: c++ -*- */
#ifndef __AlibavaGUI_h__
#define __AlibavaGUI_h__

#include <AlibavaConfig.h>
#include <GladeApp.h>
#include <Alibava.h>
#include <Analysis.h>
#include <DataFile.h>
#include <KeyFile.h>
#include <gtkhst/histogram.h>
#include <gtkhst/tracer.h>
#include <DAQmanager.h>
#include <string>
#include <deque>
#include <map>

class AlibavaGUI : public GladeApp, public DAQmanager, public DataMonitor
{
    private:
        // The Alibava object
        // Alibava _board;


        // Gtk signal handlers
        static GtkWidget *create_histogram(AlibavaGUI *, char *name, char *s1, char *s2, int i1, int i2);

        static void on_open(GtkWidget *, AlibavaGUI *);
        static void on_save(GtkWidget *, AlibavaGUI *);
        static void on_save_as(GtkWidget *, AlibavaGUI *);
        static void on_save_pedestals(GtkWidget *, AlibavaGUI *);
        static void on_load_pedestals(GtkWidget *, AlibavaGUI *);
        static void on_buttonDAQ(GtkWidget *, AlibavaGUI *);
        static void on_about(GtkWidget *, AlibavaGUI *);
        static void on_runtype(GtkWidget *, AlibavaGUI *);
        static void on_reset(GtkWidget *, AlibavaGUI *);
        static void on_calibration_config(GtkWidget *, AlibavaGUI *);
        static void on_lasersync_config(GtkWidget *, AlibavaGUI *);
        static void on_beetle_config(GtkWidget *, AlibavaGUI *);
        static void on_laser_config(GtkWidget *, AlibavaGUI *);
        static void on_trigger_config(GtkWidget *, AlibavaGUI *);
        static void on_log_data(GtkWidget *, AlibavaGUI *);
        static void on_daq_config(GtkWidget *, AlibavaGUI *);
        static void on_analysis_config(GtkWidget *, AlibavaGUI *);
        static void on_data_type(GtkWidget *, AlibavaGUI *);
        static void on_signal_select(GtkWidget *, AlibavaGUI *);
        static void on_show_headers(GtkWidget *, AlibavaGUI *);
        static void on_time_profile(GtkWidget *, AlibavaGUI *);
        static void on_reset_hst(GtkWidget *, AlibavaGUI *);
        static void on_help_activate(GtkWidget *, AlibavaGUI *);
        static void on_plugin_activate(GtkWidget *, AlibavaGUI *);
        static void on_plugin_menu_activate(GtkWidget *, AlibavaGUI *);

        static gboolean update_labels(AlibavaGUI *);
        static gboolean send_message(AlibavaGUI *);

        void reset_hst();

        GTimer *_timer;

        gpointer _thread;
        HstHistogram *current_hst, *hitmap_hst, *event_hst;
        HstTracer *temp_hst, *time_hst, *noise_tracer[2], *cmmd_tracer[2];

        bool _show_time_profile;
        int _cntxt;
        std::deque<std::string> _msg;
        std::string sfile;
        std::map<std::string, GtkWidget *> hst_list;

        void unref_histograms(GObject *);
        void create_time_profiles(GObject *);

        void set_run_manager( RunManager::RunType type);
        int check_event_headers(const EventData* b);

    public:
        AlibavaGUI(int nevts, Alibava *B);
        ~AlibavaGUI();

        // GladeApp stuff
        void user_init();
        void set_handlers();
        void auto_connect();
        void process_text(const std::string &s, GtkWidget *w);

        int prepare_for_run();
        void process_event(const EventData *);
        static gpointer acquire_thrd(AlibavaGUI *, bool is_thread=true);


        Analysis &analysis();
        void nevts(int x);
        void sample_size(int x);

        void add_message(const char *msg);

        void create_signal_profile(const char *, int nb, double xmin, double xmax);
        void create_signal_histogram();

        void getHistogram(const std::string &whst, size_t &size, char **buffer);
        void set_debug(bool x);
};


#endif
