/*
 * RunManager.cc
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */

#include <string>
#include "RunManager.h"
#include <LaserSyncRun.h>
#include <CalibrationRun.h>
#include <PedestalRun.h>
#include <RadSourceRun.h>
#include <LaserRun.h>
#include <DataFile.h>
#include <DAQmanager.h>

RunManager *RunManager::factory(RunManager::RunType type, DAQmanager *daqm)
{
    RunManager *rm = 0;
    switch (type) {
        case Calibration:
            rm = new CalibrationRun(daqm);
            break;
        case LaserSync:
            rm = new LaserSyncRun(daqm);
            break;
        case Pedestal:
            rm = new PedestalRun(daqm);
            break;
        case Laser:
            rm = new LaserRun(daqm);
            break;
        case RadSource:
            rm = new RadSourceRun(daqm);
            break;
        default:
            break;
    }
    return rm;
}

RunManager::RunManager(DAQmanager *D)
: daqm(D), _plugin(new Plugin()) {}


void RunManager::set_plugin(Plugin *p)
{
    if ( _plugin == p )
        return;

//    TODO: check the logic of how plugins are loaded and unloaded
//    if ( _plugin )
//        delete _plugin;

    _plugin = p;
}


void RunManager::check_new_event(int ievt)
{
    if ( plugin() &&  plugin()->new_event(ievt) )
    {
        std::string out = plugin()->new_point();
        if (!out.empty() && daqm->out_file())
            daqm->out_file()->write(DataFile::CheckPoint, out.size(), out.c_str());
    }
}
////
