/* -*- mode: c++ -*- */
#ifndef __ScanConfig_h__
#define __ScanConfig_h__

#include <ConfigFile.h>

class ScanConfig : public ConfigFile
{
    private:
        unsigned int _from;
        unsigned int _to;
        unsigned int _npts;
        unsigned int _nevts;
        unsigned int _min_step;

    protected:
        void cpy(const ScanConfig &c);
    public:
        ScanConfig(unsigned int np=1, unsigned int first=0, unsigned int last=1, unsigned int min_step=1) :
            _from(first), _to(last), _npts(np), _nevts(100), _min_step(min_step)
        {
            adjust();
        }
        ScanConfig(const ScanConfig &c)
        {
            cpy(c);
        }
        ScanConfig &operator=(const ScanConfig &c)
        {
            if ( this != &c)
                cpy(c);

            return *this;
        }
        void adjust();

        unsigned int from() const
        {
            return _from;
        }
        unsigned int to() const
        {
            return _to;
        }
        unsigned int npts() const
        {
            return _npts;
        }
        unsigned int step() const
        {
            return (_to-_from)/_npts;
        }
        void set(unsigned int np, unsigned int first, unsigned int last)
        {
            _from = first;
            _to = last;
            _npts = np;
            adjust();
        }
        void from(unsigned int x)
        {
            _from = x;
        }
        void to(unsigned int x)
        {
            _to = x;
        }
        void npts(unsigned int x)
        {
            _npts = x;
        }
        unsigned int nevts() const
        {
            return _nevts;
        }
        void nevts(unsigned int n)
        {
            _nevts = n;
        }
        void set_npts(int step)
        {
            _npts = (_to - _from)/step;
            adjust();
        }

        void set_min_step(int x)
        {
            _min_step = x;
        }

        void load_data();
        void save_data();
};

class CalibrationConfig : public ScanConfig
{
    private:
        unsigned short _delay;
        unsigned int _charge;
        bool _scan_charge;
    public:
        CalibrationConfig(unsigned int np=10, unsigned int first=0, unsigned int last=10, unsigned short delay=85) :
            ScanConfig(np, first, last, 1025), _delay(delay), _charge(30000), _scan_charge(true)
        {
            set_group("calibration");
        }
        CalibrationConfig(const CalibrationConfig &c);
        CalibrationConfig &operator=(const CalibrationConfig &c);

        void load_data();
        void save_data();

        unsigned short delay() const { return _delay; }
        void delay(unsigned short v) { _delay=v; }

        unsigned int charge() const { return _charge; }
        void charge(unsigned int v) { _charge = v; }

        bool scan_charge() const  { return _scan_charge; }
        void scan_charge(bool v) { _scan_charge = v; }
};

class LaserSyncConfig : public ScanConfig
{
    public:
        LaserSyncConfig(unsigned int np=10, unsigned int first=100, unsigned int last=500) :
            ScanConfig(np, first, last)
        {
            set_group("laser_sync");
        }
};

#endif
