#ifndef UTILS_H_
#define UTILS_H_

#include <string>
#include <vector>
#include <cstdio>
#include <cerrno>
#include <cstring>
#include <glib.h>
#include <glib.h>

#ifdef HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif
/** TODO: Use this properly
 *        According to the "manual"....
 *        In order to use these macros in an application, 
 *        you must include glib/gi18n.h. For use in a library, 
 *        must include glib/gi18n-lib.h after defining the 
 *        GETTEXT_PACKAGE macro suitably for your library.
 *        
 *        Later on it adds: 
 *        Note that you also have to call setlocale() and textdomain() 
 *        (as well as bindtextdomain() and bind_textdomain_codeset()) 
 *        early on in your main() to make gettext() work.
 *        
 *        I believe I do the latest.
 */
/*
 * Standard gettext macros.
 */
#if defined(ENABLE_NLS) || defined(USE_INTLTOOL)
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  define Q_(String) g_strip_context ((String), gettext (String))
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define Q_(String) g_strip_context ((String), (String))
#  define N_(String) (String)
#endif


char *get_time();

#ifndef g_debug
#define g_debug(...)    g_log (G_LOG_DOMAIN,         \
                               G_LOG_LEVEL_DEBUG,    \
                               __VA_ARGS__)

#endif




#if defined(ASCII)

void p_debug(const gchar *func_name, const gchar *fmt, ...);
void p_err_msg(const gchar *func_name, int func_line, const gchar *format,...);
void p_info_msg(const gchar *func_name, int func_line, const gchar *format,...);
void write_msg(const gchar *fmt, ...);

#if defined(DEBUG)
#define PDEBUG(fmt, args...) p_debug(__FUNCTION__, fmt, ## args )
#else
#define PDEBUG(...)
#endif

#define err_msg_ex(func, fmt, args...) \
    p_err_msg(func, __LINE__, fmt, ## args)

#define err_msg(fmt, args...) \
    p_err_msg(__PRETTY_FUNCTION__, __LINE__, fmt, ## args)

#define info_msg_ex(func, fmt, args...) \
    p_info_msg(func, __LINE__, fmt, ## args)

#define info_msg(fmt, args...) \
    p_info_msg(__PRETTY_FUNCTION__, __LINE__, fmt, ## args)


#else /* ASCII */
void __print_msg(const char *, int error=0);
void set_text_buffer(char *ptr, char *adj);
void x_debug(const gchar *func_name, const gchar *fmt, ...);
void x_err_msg(const gchar *func_name, int func_line, const gchar *format,...);
void x_info_msg(const gchar *func_name, int func_line, const gchar *format,...);
void write_msg(const gchar *fmt, ...);

#ifndef PDEBUG
#if defined(DEBUG)
#define PDEBUG(fmt, args...) x_debug(__FUNCTION__ , fmt , ## args);
#else
#define PDEBUG(...)
#endif
#endif


#define err_msg(fmt, args...) \
        x_err_msg(__PRETTY_FUNCTION__, __LINE__, fmt, ##args)

#define info_msg(fmt, args...) \
    x_info_msg(__PRETTY_FUNCTION__, __LINE__, fmt, ##args)

#define err_msg_ex(func, fmt, args...) \
    x_err_msg(func, __LINE__, fmt, ## args)

#define info_msg_ex(func, fmt, args...) \
    x_info_msg(func, __LINE__, fmt, ##args)


#endif /* ASCII */

extern unsigned int __verbose__;
void p_trace_msg(const gchar *func_name, int func_line, bool in, const gchar *format,...);
#define ALIBAVA_TRACE_IN(fmt, args...) \
    if (__verbose__) \
        p_trace_msg(__PRETTY_FUNCTION__, __LINE__, true, fmt, ## args)

#define ALIBAVA_TRACE_OUT(fmt, args...) \
    if (__verbose__) \
        p_trace_msg(__PRETTY_FUNCTION__, __LINE__, false, fmt, ## args)


class Verbose
{
    private:
        static std::vector<std::string> _backtrace;
        static void push_call(const std::string &func, int line);
        static void pop_call();
        std::string _msg, _func;
        int  _line;
    public:
        Verbose(const std::string &msg, const char *func, int line);
        ~Verbose();
        static void backtrace();
};
#define ALIBAVA_TRACE_CLASS(msg) \
    Verbose ___rc___ = Verbose(msg, __PRETTY_FUNCTION__, __LINE__)

#endif /*UTILS_H_*/
