/*
 * test_plugin.cc
 *
 * This is an example of a plugin writen in C++.
 * Look at the documentation in PLugin.h
 *
 *  Created on: Jul 24, 2009
 *      Author: lacasta
 */
#include <iostream>
#include <sstream>
#include <Plugin.h>
#include "NewPoint.h"

/**
 * This is an implementation of the Plugin class. It is a simple example
 * that will make alibava-gui make a scan. We may use the new_file method
 * to store the parameters of the scan, new_event to determine when a new
 * point is the scan is needed and new_point to store the actual values of
 * the scan variables.
 */
class MyPlugin : public Plugin
{
    private:
        int npoints;          // Number of points we want for the scan
        int nevt_per_point;   // Number of point to be acquired for each point
        int run_type;         // type of run
        int current_event;    // current event number
        EventCntr handler;    // The object that decides when to change to
                              // the next point

    public:
        // Constructor with default values
        MyPlugin() :
            npoints(50), nevt_per_point(1000), run_type(-1),
            handler(nevt_per_point), current_event(0) {}

        // destructor
        ~MyPlugin() {}

        /**
         * Declaration of Plugin methods to be implemented
         */
        std::string new_file();
        int start_of_run(int run_type, int nevts, int sample_size);
        std::string end_of_run();
        bool new_event(int evt);
        std::string new_point();
};

class MyPluginFilter: public MyPlugin
{
    private:
        int nchan;
        int *chan_list;
    public:
        MyPluginFilter() :
            nchan(5), chan_list(0)
        {
            int i;
            chan_list = new int[nchan];
            for (i=0;i<nchan;i++)
                chan_list[i] = 5*i + 3;
        }
        ~MyPluginFilter() {}
        std::string filter_event(const EventData *data);
};

std::string MyPlugin::new_file()
{
    std::string rc("New file");
    std::cout << "new_file" << std::endl;
    return rc;
}

int MyPlugin::start_of_run(int runtype, int nevts, int sample_size)
{
    run_type = runtype;
    std::cout << "start_of_run " << nevts << " events. "
              << "Runtype " << run_type
              << std::endl;

    if (sample_size > handler.value())
    {
        handler.value(sample_size);
        nevt_per_point = sample_size;
    }
    handler.reset();
    return npoints*nevt_per_point;
}

std::string MyPlugin::end_of_run()
{
    std::string rc("end_of_run");
    std::cout << "end_of_run" << std::endl;
    return rc;
}

bool MyPlugin::new_event(int ievt)
{
    current_event = ievt;
    return handler(ievt);
}

std::string MyPlugin::new_point()
{
    std::ostringstream ostr;
    ostr << "new point: " << current_event << "\tval1\tval2\tval3";
    std::cout << ostr.str() << std::endl;
    return ostr.str();
}

std::string MyPluginFilter::filter_event(const EventData *data)
{
    int ichan;
    std::ostringstream ostr;
    const EventBlock *B = data->get_block();
    ostr.write((char *)&B->time, sizeof(int));
    ostr.write((char *)&B->temp, sizeof(short));
    for (ichan=0;ichan<nchan; ichan++)
    {
        int ichip = chan_list[ichan]/128;
        int jchan = ichan % 128;
        ostr.write((char *)B->chip[ichip].signal[jchan], sizeof(short));
    }
    return ostr.str();
}

extern "C"
{
    Plugin *create_plugin()
    {
        MyPlugin *plugin = new MyPlugin();
        return plugin;
    }

    Plugin *create_plugin_filter()
    {
        MyPluginFilter *plugin = new MyPluginFilter();
        return plugin;
    }
}
